﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Newtonsoft.Json;
namespace ConsoleApplication2
{
    class Program
    {
        static void Main(string[] args)
        {
            //读取xml文本内容
            var xml = File.ReadAllText(@".//cnSingle.xml");
            //反序列化成对象
            var cnblog = Deserialize<Rss>(xml);
            //序列化成xml字符串
            xml = Serialiaze(cnblog);

            //DataTable 序列化与序列化
            DataTable dt = new DataTable();
            dt.TableName = "TestTab";
            dt.Columns.Add("ID", typeof(int));
            dt.Columns.Add("Value", typeof(string));
            for (var i = 0; i < 10; i++)
            {
                var dr = dt.NewRow();
                dr["ID"] = i + 1;
                dr["Value"] = DateTime.Now.Ticks.ToString();
                dt.Rows.Add(dr);
            }
            //这样序列化DataTable得到的Xml不够简洁，关于DataTable与Xml互转 提供另外一种解决方案
            xml = Serialiaze(dt);
            dt = Deserialize<DataTable>(xml);
            //将DataTable与Xml 互转
            var xmlNew = DataToXml(dt);
            dt = XmlToDataTable(xmlNew);
        }

        /// <summary>
        /// 序列化
        /// </summary>
        public static string Serialiaze(object obj)
        {
            StringBuilder xml = new StringBuilder();
            using (XmlWriter writer = XmlWriter.Create(xml))
            {
                XmlSerializer xs = new XmlSerializer(obj.GetType());
                xs.Serialize(writer, obj);
                return xml.ToString();
            }
        }
        /// <summary>
        /// 反序列化
        /// </summary>
        public static T Deserialize<T>(string xmlContent)
        {
            XmlSerializer xs = new XmlSerializer(typeof(T));
            using (StringReader strReader = new StringReader(xmlContent))
            {
                XmlReader xmlReader = XmlReader.Create(strReader);
                return (T)xs.Deserialize(xmlReader);
            }
        }

        #region 将DataTable与Xml 互转
        /// <summary> 
        /// 将DataTable对象转换成XML字符串 
        /// </summary> 
        /// <param name="dt">DataTable对象</param> 
        /// <returns>XML字符串</returns> 
        public static string DataToXml(DataTable dt)
        {
            if (dt != null)
            {
                MemoryStream ms = null;
                XmlTextWriter XmlWt = null;
                try
                {
                    ms = new MemoryStream();
                    //根据ms实例化XmlWt 
                    XmlWt = new XmlTextWriter(ms, Encoding.Unicode);
                    //获取ds中的数据 
                    dt.WriteXml(XmlWt);
                    int count = (int)ms.Length;
                    byte[] temp = new byte[count];
                    ms.Seek(0, SeekOrigin.Begin);
                    ms.Read(temp, 0, count);
                    //返回Unicode编码的文本 
                    UnicodeEncoding ucode = new UnicodeEncoding();
                    return ucode.GetString(temp).Trim();
                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    //释放资源 
                    if (XmlWt != null)
                    {
                        ms.Close();
                        ms.Dispose();
                    }
                }
            }
            else
            {
                return string.Empty;
            }
        }

        /// <summary> 
        /// 将Xml内容字符串转换成DataTable对象 
        /// </summary> 
        /// <param name="xmlStr">Xml内容字符串</param> 
        /// <returns>DataTable对象</returns> 
        public static DataTable XmlToDataTable(string xmlStr)
        {
            if (!string.IsNullOrEmpty(xmlStr))
            {
                StringReader StrStream = null;
                XmlTextReader Xmlrdr = null;
                try
                {
                    DataSet ds = new DataSet();
                    //读取字符串中的信息 
                    StrStream = new StringReader(xmlStr);
                    //获取StrStream中的数据 
                    Xmlrdr = new XmlTextReader(StrStream);
                    //ds获取Xmlrdr中的数据 
                    ds.ReadXml(Xmlrdr);
                    return ds.Tables[0];
                }
                catch (Exception ex)
                {
                    throw ex;
                }
                finally
                {
                    //释放资源 
                    if (Xmlrdr != null)
                    {
                        Xmlrdr.Close();
                        StrStream.Close();
                        StrStream.Dispose();
                    }
                }
            }
            else
            {
                return null;
            }
        }

        #endregion

    }

    [XmlRoot("rss")]
    public class Rss
    {
        public Channel channel { get; set; }
    }
    [XmlRoot("channel")]
    public class Channel
    {
        public string title { get; set; }
        public string link { get; set; }
        public string description { get; set; }
        public string language { get; set; }
        public string lastBuildDate { get; set; }
        public string pubDate { get; set; }
        public string ttl { get; set; }
        public Channel_Item item { get; set; }
    }

    public class Channel_Item
    {
        public string title { get; set; }
        public string link { get; set; }
        [XmlElement("dc:creator")]
        public string dccreater { get; set; }
        public string author { get; set; }
        public string pubDate { get; set; }
        public string guid { get; set; }
        public string description { get; set; }
    }

}